# -*- coding: utf-8 -*-
# !/usr/bin/python

""" abfab_entity.py: Defines an abtract ABFAB Entity """

import xml

__author__ = "Alejandro Pérez <alex.perez-mendez@jisc.ac.uk>"


class AbfabEntity(object):
    def __init__(self):
        self._attribute_filters = []

    def _get_radius_attribute(self, p, attribute_name):
        try:
            return next(value for name, value in p if name == attribute_name)
        except StopIteration:
            return None

    def _get_radius_attributes(self, p, attribute_name):
        attributes = [value for name, value in p if name == attribute_name]
        if len(attributes) == 0:
            return None
        return ''.join(attributes)

    def _get_acceptor_name(self, p):
        acceptor_service_name = self._get_radius_attribute(p, 'GSS-Acceptor-Service-Name')
        acceptor_host_name = self._get_radius_attribute(p, 'GSS-Acceptor-Host-Name')

        acceptor_name = ''
        if acceptor_service_name:
            acceptor_name = str(acceptor_service_name)
        if acceptor_host_name:
            acceptor_name += '@' + str(acceptor_host_name)
        return acceptor_name

    def _assertion_to_reply(self, assertion):
        max_rad_attr_len = 220
        assertion_str = str(assertion)
        return tuple(('SAML-AAA-Assertion', '+=', assertion_str[i:i + max_rad_attr_len])
                     for i in range(0, len(assertion_str), max_rad_attr_len))

    def _reply_to_assertion(self, p):
        # Parse assertion. In case of problems, return an empty Assertion
        return self._get_radius_attributes(p, 'SAML-AAA-Assertion')

    def _attributes_to_statement(self, attributes):
        statement = '<ns0:AttributeStatement xmlns:ns0="urn:oasis:names:tc:SAML:2.0:assertion" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xs="http://www.w3.org/2001/XMLSchema">'
        for name, name_format, values in attributes:
            saml_attribute = '<ns0:Attribute Name="{0}" NameFormat="{1}">'.format(name, name_format)
            # add attribute values
            for value in values:
                # convert to base64 if not in utf8
                try:
                    unicode(value)
                except (UnicodeDecodeError, TypeError):
                    value = value.encode('base64')
                # This is required to work with Python3
                except NameError:
                    pass
                saml_attribute += '<ns0:AttributeValue xsi:type="xs:string">{0}</ns0:AttributeValue>'.format(value)
            saml_attribute += "</ns0:Attribute>"
            statement += saml_attribute
        statement += "</ns0:AttributeStatement>"
        return statement

    # replace assertion's attribute statements with the indicated attributes
    def _replace_assertion_attributes(self, assertion, attributes):
        root = xml.etree.ElementTree.fromstring(assertion)
        root.remove(root.find("./{urn:oasis:names:tc:SAML:2.0:assertion}AttributeStatement"))
        newst = xml.etree.ElementTree.fromstring(self._attributes_to_statement(attributes))
        root.append(newst)
        return xml.etree.ElementTree.tostring(root)

    def _assertion_to_attributes(self, assertion):
        root = xml.etree.ElementTree.fromstring(assertion)
        attributes = []
        for attribute in root.findall("./{urn:oasis:names:tc:SAML:2.0:assertion}AttributeStatement/"):
            attribute = attribute.attrib['Name'], attribute.attrib['NameFormat'], [str(x.text) for x in attribute]
            attributes.append(attribute)
        return attributes

    def _perform_filters(self, attributes, rp_name, idp_name):
        for attr_filter in self._attribute_filters:
            attributes = attr_filter.filter_attributes(attributes, rp_name=rp_name,
                                                       idp_name=idp_name)
        return attributes
